use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHLIntraship::API::SOAP::Parser;
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;
use Test::Deep;
use Test::Exception;
use Test::More;

## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProtectPrivateVars)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            local *DE_EPAGES::DHLIntraship::API::DeveloperAuthentification::GetAuthorKeys = sub{
                return {
                    'DEVID'  => 'DEVID',
                    'APPID'  => 'APPID',
                    'CERTID' => 'CERTID'
                };
            };
            testParseCreateShippmentResponseOk();
            testParseCreateShippmentResponseStatusError();
            testParserCreateShippmentStatusArray();
            #testParseCreateShippmentResponseErrorCode();
            done_testing();
        }
    );
}

sub testParseCreateShippmentResponseOk {

    my $WSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(0);
    $WSClient->{'TransportHook'} = \&createShippmentDD;
    my $mockSL = MockObjectPM();
    my $returnHash = $WSClient->createShipmentDD($mockSL);

    is_deeply($returnHash, [
          {
            'StatusCode' => '0',
            'ShipmentNumber' => '00340433836000191469',
            'SequenceNumber' => '1',
            'InvalidAddress' => '',
            'Labelurl' => 'http://test-intraship.dhl.com:80/cartridge.55/WSPrint?code=2DF3501616A182E9328C91C83B00509C4A611A39324EAA66',
            'StatusMessage' => 'ok'
          }
        ], 'Check if the parser is returning the correct values');
}

sub testParseCreateShippmentResponseStatusError {

    my $WSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(0);
    $WSClient->{'TransportHook'} = \&createShippmentDDStatusError;
    my $mockSL = MockObjectPM();
    throws_ok { $WSClient->createShipmentDD($mockSL) } qr/DHLRespondedWithError/, 'DHL Responded With Error Status. A error is thrown.';
}

sub testParseCreateShippmentResponseErrorCode {

    my $WSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(0);
    $WSClient->{'TransportHook'} = \&createShippmentDDErrorCode;
    my $mockSL = MockObjectPM();
    throws_ok { $WSClient->createShipmentDD($mockSL) } qr/DHLRespondedWithError/, 'HTML error code recieved. A error is thrown.';
}

sub createShippmentDD($$) {
    my ($request, $trace) = @_;

    HTTP::Response->new(200, 'Constant' ## no critic (ProhibitMagicNumbers)
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
    <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
    <soapenv:Body>
       <ns2:CreateShipmentResponse xmlns:ns2="http://de.ws.intraship">
          <Version xmlns="http://dhl.de/webservice/cisbase">
             <majorRelease>1</majorRelease>
             <minorRelease>0</minorRelease>
             <build>11</build>
          </Version>
          <status>
             <StatusCode>0</StatusCode>
             <StatusMessage>ok</StatusMessage>
          </status>
          <CreationState>
             <StatusCode>0</StatusCode>
             <StatusMessage>ok</StatusMessage>
             <SequenceNumber>1</SequenceNumber>
             <ShipmentNumber>
                <ns1:shipmentNumber xmlns:ns1="http://dhl.de/webservice/cisbase">00340433836000191469</ns1:shipmentNumber>
             </ShipmentNumber>
             <PieceInformation>
                <PieceNumber>
                   <ns1:licensePlate xmlns:ns1="http://dhl.de/webservice/cisbase">00340433836000191469</ns1:licensePlate>
                </PieceNumber>
             </PieceInformation>
             <Labelurl>http://test-intraship.dhl.com:80/cartridge.55/WSPrint?code=2DF3501616A182E9328C91C83B00509C4A611A39324EAA66</Labelurl>
          </CreationState>
       </ns2:CreateShipmentResponse>
    </soapenv:Body>
    </soapenv:Envelope>
__ANSWER
}

sub createShippmentDDStatusError($$) {
    my ($request, $trace) = @_;

    HTTP::Response->new(200, 'Constant' ## no critic (ProhibitMagicNumbers)
      , [ 'Content-Type' => 'text/xml' ]
      , <<__ANSWER)
<?xml version=\'1.0\' encoding=\'utf-8\'?><soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"><soapenv:Body><ns4:CreateShipmentResponse xmlns:ns4="http://de.ws.intraship"><Version xmlns="http://dhl.de/webservice/cisbase"><majorRelease>1</majorRelease><minorRelease>0</minorRelease><build>14</build></Version><status><StatusCode>1000</StatusCode><StatusMessage>SERVICE IS TEMPORARY NOT AVAILABLE</StatusMessage></status></ns4:CreateShipmentResponse></soapenv:Body></soapenv:Envelope>
__ANSWER
}

sub createShippmentDDErrorCode($$) {
    my ($request, $trace) = @_;

    HTTP::Response->new(404); ## no critic (ProhibitMagicNumbers)
}

sub testParserCreateShippmentStatusArray{
    my $OperationName ='createShipmentDD';
    my $SOAPResponse = {
      'part1' => {
        'CreationState' => [
          {
            'Labelurl' => 'http://test-intraship.dhl.com:80/cartridge.591/WSPrint?code=CDDC53AA015C1A47691A83C7ED76A9455076E3DC96CDBF82',
            'PieceInformation' => [
              'HASH(0x13e86f50)'
            ],
            'SequenceNumber' => '22452',
            'ShipmentNumber' => {
              'shipmentNumber' => '00340433836004950390'
            },
            'StatusCode' => '0',
            'StatusMessage' => [
              'ok',
              'Warning: The address could not be validated correctly. Please correct the street name, because this is unknown for entered post code or written incorrect. This could lead to delays within shipment delivery advice ([NON_CODABLE])',
              'Warning: The address could not be validated correctly. Please correct the street name, because this is unknown for entered post code or written incorrect. This could lead to delays within shipment delivery advice ([NON_CODABLE])'
            ]
          }
        ],
        'Version' => {
          'build' => '14',
          'majorRelease' => '1',
          'minorRelease' => '0'
        },
        'status' => {
          'StatusCode' => bless( {
            'sign' => '+',
            'value' => [
              '0'
            ]
          }, 'Math::BigInt' ),
          'StatusMessage' => 'ok'
        }
      }
    };

    my $Parser = DE_EPAGES::DHLIntraship::API::SOAP::Parser->new();
    my $returnValue = $Parser->parseResponse($OperationName, $SOAPResponse);
    is_deeply($returnValue, [
          {
            'StatusCode' => '0',
            'ShipmentNumber' => '00340433836004950390',
            'SequenceNumber' => '22452',
            'InvalidAddress' => 1,
            'Labelurl' => 'http://test-intraship.dhl.com:80/cartridge.591/WSPrint?code=CDDC53AA015C1A47691A83C7ED76A9455076E3DC96CDBF82',
            'StatusMessage' => 'ok'
          }
        ], 'Check if the parser is returning the correct values');
}

sub MockObjectPM {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages RequireCommentForExportedFunctions )
        package DE_EPAGES::Address::API::Object::Address;
        sub id{return 197;}
        sub deleteChildren { return; };
        sub folder {my $self = shift; return $self; };
        sub children {my $self = shift; return $self->{'Folder'}; };
        sub alias {my $self = shift; return $self->{'Alias'} //'Alias';};
        sub insertChild {
            my $self = shift;
            my ($Class, $hInfo) = @_;
            my $obj = {};
            $obj->{'Class'}         = $Class;
            $obj->{'Alias'}         = $hInfo->{'ServiceName'};
            $obj->{'ServiceGroup'}  = $hInfo->{'ServiceGroup'};
            $obj->{'ServiceAmount'} = $hInfo->{'ServiceAmount'};
            bless $obj, 'DE_EPAGES::Address::API::Object::Address';
            push(@{$self->{'Folder'}}, $obj);
        };
        sub get {
            my $self = shift;
            my ($aNames) = @_;
            if(ref($aNames) ne 'ARRAY'){
                return $self->_get($aNames);
            }

            my $hInfo = {};
            foreach my $Name (@{$aNames}) {
                $hInfo->{$Name} = $self->_get($Name);
            }

            return $hInfo;
        }
        sub _get{
            my $self = shift;
            my ($name) = @_;
            if ($name ~~ [qw(LineItemContainer ShopAddress Address Country ShippingFulfillmentMethod)]) {
                return $self;
            }
            elsif ($name ~~ [qw( LineItemsSum TotalWeight PhoneCell Phone HouseNumber )]) {
                return 1;
            }
            elsif ($name eq 'Zipcode')
            {
                # it is a Zipcode
                return 10537; ## no critic (ProhibitMagicNumbers RequireNumberSeparators)
            }
            elsif ($name eq 'CustomerNumber') {
                #its a random number with 14 digits
                return 12345678900176; ## no critic (ProhibitMagicNumbers RequireNumberSeparators)
            }
            elsif ($name eq 'EstimatedDeliveryDate' )
            {
                my $dt = DateTime->new(
                year       => 2222,
                month      => 2,
                day        => 22,
                hour       => 22,
                minute     => 22,
                );
                return $dt;
            }
            else
            {
                return $name;
            }
        }
        sub findChild {
            my $self = shift;
            my ($alias) = @_;
            my ($returnValue) =  grep {$_->{'Alias'} eq $alias}  @$self;
            return $returnValue;

        }
        return bless {'Folder' => [], 'Code2' => 'DE' }, 'DE_EPAGES::Address::API::Object::Address';
    }
}

## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitPrivateSubroutineUse ProtectPrivateVars RequireCommentForExportedFunctions)

RunScript( Sub => \&Main );